# -*- coding: utf-8 -*-

"""External Datalore web application."""

__all__ = ['DataloreApp']

import typing
from anaconda_navigator.static import images
from . import base

if typing.TYPE_CHECKING:
    from anaconda_navigator.config import user as user_config


class DataloreApp(base.BaseWebApp):  # pylint: disable=too-few-public-methods
    """Shortcut for JetBrains Datalore web application."""

    def __init__(self, config: 'user_config.UserConfig') -> None:
        """Initialize new :class:`DataloreApp` instance."""
        super().__init__(
            app_name='datalore',
            display_name='Datalore',
            description=(
                'Online Data Analysis Tool with smart coding assistance by JetBrains. Edit and run your Python '
                'notebooks in the cloud and share them with your team.'
            ),
            image_path=images.DATALORE_ICON_1024_PATH,
            url='http://www.anaconda.com/datalore_navigator',
            is_available=True,
            config=config,
        )
