#!/opt/concourse/worker/volumes/live/a76d46df-8008-46e2-763c-d3f400318082/volume/anaconda-navigator_1648477386214/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/a76d46df-8008-46e2-763c-d3f400318082/volume/anaconda-navigator_1648477386214/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])
